/***************************************************************************
**    xDefaults.cpp  $Revision: 1.4 $ - $Name:  $ 
**    Reads Defaults file
**
**    Copyright (C) 1996 Joseph Croft <jcroft@unicomp.net>
**
**    This library is free software; you can redistribute it and/or
**    modify it under the terms of the GNU Library General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    This library is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**    Library General Public License for more details.
**
**    You should have received a copy of the GNU Library General Public
**    License along with this library; if not, write to the Free
**    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include "xMisc.h"
#include "xDefaults.h"

static int dbg = 0;

static char escTable[256];

xDefltEntry::xDefltEntry(const char *pTag, const char *pValue)
{
   tag = "";
   while (*pTag)
   {
      if (*pTag != '\r' && *pTag != '\n')
         tag += toupper(*pTag);
      pTag++;
   }
   value = pValue;
   pNext = pPrev = NULL;
}

xDefltEntry::~xDefltEntry()
{
   unlink();
}

bool xDefltEntry::is(const char *pTag)
{
   QString tmpTag;
   
   while (*pTag)
   {
      if (*pTag != '\r' && *pTag != '\n')
         tmpTag += toupper(*pTag);
      pTag++;
   }
   return(tag == tmpTag);
}

const char *xDefltEntry::val(const char *pVal)
{
   const char *rv;
   
   rv = (const char *)value;
   value = pVal;
   return(rv);
}

const char *xDefltEntry::val(xDefltEntry *pDefaults, bool expand)
{
   const char *rv;

   if (dbg) fprintf(stdout, "xDefltEntry::val():Expand = %d\n", expand);
   if (expand == TRUE)
   {
      strReplaceEnvArgs(pDefaults, value, expanded);
      rv = expanded;
   }
   else
      rv = value;
   return(rv);
}

void xDefltEntry::strReplaceEnvArgs(xDefltEntry *pDefaults, QString &value,
                                     QString &strBuf)
{
   xDefltEntry *pEntry;
   char ch, *cpe, ebuf[80];
   const char *cp, *cp2;
   
   strBuf = "";
   for (cp = (const char *)value; *cp != '\0'; cp++)
   {
      if (*cp == '$')
      {
         for (cpe = ebuf, cp++; !ispunct(*cp) && !isspace(*cp) && *cp != '\0'; cp++)
         {
            *(cpe++) = *cp;
            *cpe = '\0';
         }
         cp--;
         if (dbg) fprintf(stdout, "Found Tag: %s\n", ebuf);
         if (dbg) fflush(stdout);
         for (cp2 = NULL, pEntry = pDefaults; 
              pEntry != NULL && pEntry; 
              pEntry = pEntry->next())
         {
            if (pEntry->is(ebuf))
            {
               if (dbg) fprintf(stdout, "1:Tag is a Macro: %s\n", ebuf);
               if (dbg) fflush(stdout);
               cp2 = pEntry->val(pDefaults);
               break;
            }
         }
         if (cp2 != NULL)
         {
            if (dbg) fprintf(stdout, "2:Tag is Macro: %s\n", ebuf);
            if (dbg) fflush(stdout);
            strBuf += cp2;
            if (dbg) fprintf(stdout, "Replacing with |%s|\n", cp2);
            if (dbg) fflush(stdout);
         }
         else if ((cp2 = getenv(ebuf)) != NULL)
         {
            if (dbg) fprintf(stdout, "Tag is Environment Variable: %s\n", ebuf);
            if (dbg) fflush(stdout);
            strBuf += cp2;
            if (dbg) fprintf(stdout, "Replacing with |%s|\n", cp2);
            if (dbg) fflush(stdout);
         }
         else
         {
            if (dbg) fprintf(stdout, "  -Not defined\n");
            if (dbg) fflush(stdout);
            continue;
         }
      }
      else if (*cp == '\\')
      {
         if (dbg) fprintf(stdout, "Found escaped char %x\n", *(cp + 1));
         if (*(cp + 1))
         {
            if (toupper(*(cp + 1)) == 'X' || isdigit(*(cp + 1)))
               ch = strtoc(cp, &cp);
            else if (escTable[*(++cp)] != 0)
               ch = escTable[*cp];
            else
               ch = *(cp++);
            strBuf += ch;
         }
      }
      else
         strBuf += *cp;
   }
}

xDefaults::xDefaults()
{
   pDefaults = NULL;
}

xDefaults::~xDefaults()
{
   xDefltEntry *pEntry, *pEntry1;
   
   for (pEntry = pDefaults; pEntry != NULL; pEntry = pEntry1)
   {
      pEntry1 = pEntry->next();
      delete pEntry;
   }
}

xDefltEntry *xDefaults::find(const char *pTag)
{
   xDefltEntry *pEntry;
   
   if (dbg) fprintf(stdout, "xDefaults::find():Enter\n");
   if (dbg) fflush(stdout);
   for (pEntry = pDefaults; pEntry != NULL; pEntry = pEntry->next())
   {
      if (dbg) fprintf(stdout, "xDefaults::find():Testing tag = %s\n", (const char *)pEntry->tag);
      if (dbg) fflush(stdout);
      if (pEntry->is(pTag))
         break;
   }
   if (dbg) fprintf(stdout, "xDefaults::find():Exit(%d)\n", (int)pEntry);
   if (dbg) fflush(stdout);
   return(pEntry);
}

xDefltEntry *xDefaults::add(const char *pTag, const char *pVal)
{
   xDefltEntry *pEntry, *pEntry1;
   
   if (dbg) fprintf(stdout, "xDefaults::Add():Enter\n");
   if (dbg) fflush(stdout);
   if ((pEntry = find(pTag)) == NULL)
   {
      if (dbg) fprintf(stdout, "xDefaults::Add():Adding new entry\n");
      if (dbg) fflush(stdout);
      if ((pEntry = new xDefltEntry(pTag, pVal)) != NULL)
      {
         if (dbg) fprintf(stdout, "xDefaults::Add():seeking end of list\n");
         if (dbg) fflush(stdout);
         for (pEntry1 = pDefaults; 
              pEntry1 != NULL && pEntry1->next() != NULL; 
              pEntry1 = pEntry1->next());
         if (pEntry1 != NULL)
         {
            if (dbg) fprintf(stdout, "xDefaults::Add():Adding entry to the end of list\n");
            if (dbg) fflush(stdout);
            pEntry->link(pEntry1);
         }
         else
         {
            if (dbg) fprintf(stdout, "xDefaults::Add():Adding first entry of list\n");
            if (dbg) fflush(stdout);
            pDefaults = pEntry;
         }
      }
   }
   else
   {
      if (dbg) fprintf(stdout, "xDefaults::Add():Modifying existing entry\n");
      if (dbg) fflush(stdout);
      pEntry->val(pVal);
   }
   if (dbg) fprintf(stdout, "xDefaults::Add():Exit\n");
   if (dbg) fflush(stdout);
   return(pEntry);
}

const char *xDefaults::set(const char *pTag, const char *pVal)
{
   xDefltEntry *pEntry;
   const char *rv;
      
   if (dbg) fprintf(stdout, "SetDefault():Enter\n");
   if (dbg) fflush(stdout);
   if ((pEntry = find(pTag)) != NULL)
      rv = (const char *)pEntry->val(pDefaults);
   else
      rv = (const char *)NULL;
   add(pTag, pVal);
   return(rv);
}

const char *xDefaults::get(const char *pTag, bool expand)
{
   xDefltEntry *pEntry;
   const char *rv;
      
   if (dbg) fprintf(stdout, "get(\"%s\"):Enter, expand = %d\n", pTag, expand);
   if (dbg) fflush(stdout);
   if ((pEntry = find(pTag)) != NULL)
      rv = pEntry->val(pDefaults, expand);
   else
      rv = (const char *)NULL;
   if (dbg) fprintf(stdout, "get():Almost Exit\n");
   if (dbg) fflush(stdout);
   if (dbg) fprintf(stdout, "get():Exit(%s)\n", rv);
   if (dbg) fflush(stdout);
   return(rv);
}

bool xDefaults::load(FILE *pFile, char *pDefaults[])
{
   int x, rv;
   char tagBuf[80], valBuf[4096], *cp, ch;
   
   if (dbg) fprintf(stdout, "xDefaults::load()Enter\n");
   if (dbg) fflush(stdout);
   rv = 1;
   if (pDefaults != NULL)
   {
      for (x = 0; pDefaults[x] != NULL; x++)
      {
         strcpy(valBuf, pDefaults[x]);
         if (dbg) fprintf(stdout, "Read new default: |%s|\n", valBuf);
         if (dbg) fflush(stdout);
         for (cp = valBuf; *cp; cp++)
         {
            if (*cp == '\r' || *cp == '\n')
               *cp = '\0';
         }
         
         if (dbg) fprintf(stdout, "Adjusted default: |%s|\n", valBuf);
         if (dbg) fflush(stdout);
         for (cp = valBuf; *cp && *cp != '='; cp++)
         {
            if (dbg) fputc(*cp, stdout);
            if (dbg) fflush(stdout);
         }
         if (dbg) fputs("\n", stdout);
         if (dbg) fflush(stdout);
         
         if (*cp != '\0')
         {
            *(cp++) = '\0';
            if (dbg) fprintf(stdout, "Adding default to list |%s|=|%s|\n", valBuf, cp);
            if (dbg) fflush(stdout);
            if (add(valBuf, cp) == NULL)
            {
               if (dbg) fprintf(stdout, "Encountered an Error Adding default\n");
               if (dbg) fflush(stdout);
               rv = 0;
               break;
            }
         }
         else
         {
            if (dbg) fprintf(stdout, "Default not added to list\n");
            if (dbg) fflush(stdout);
         }
      }
   }
   
   ch = 0;
   for (x = 0, cp = tagBuf; pFile != (FILE *)NULL && !feof(pFile) && rv;)
   {  
      if (dbg > 2) fprintf(stdout, "xDefaults::load():x = %d, ch = 0x%x", x, ch);
      if (dbg) fflush(stdout);
      
      if (x < 5)
         ch = fgetc(pFile);
         
      switch (x)
      {
         case 0:
            if (!isspace(ch))
            {
               if (ch != '=')
               {
                  *(cp++) = ch;
                  *cp = '\0';
               }
               else
               {
                  cp = valBuf;
                  *cp = '\0';
                  x = 4;
               }
            }
            break;
            
         case 5:
         case 1:
            if (ch == '\\')
               x = 2;
            else if (ch != '\n')
            {
               if (ch == '\t')
                  ch = ' ';
               *(cp++) = ch;
               *cp = '\0';
               if (strlen(valBuf) >= sizeof(valBuf) - 1)
                  rv = 0;
               x = 1;
            }
            else
            {
               if (dbg)
               {
                  if (rv) 
                     fprintf(stdout, "\nAdjusted default: %s = %s\n", tagBuf, valBuf);
                  else
                     fprintf(stdout, "\nError parsing default %s = %s\n", tagBuf, valBuf);
                  if (dbg) fflush(stdout);
               }
               
               if (rv)
               {
                  if (dbg) fprintf(stdout, "Adding default to list- %s = %s\n", tagBuf, valBuf);
                  if (dbg) fflush(stdout);
                  if (add(tagBuf, valBuf) == NULL)
                  {
                     if (dbg) fprintf(stdout, "Encountered an Error Adding default\n");
                     if (dbg) fflush(stdout);
                     rv = 0;
                  }
               }
               x = 0;
               cp = tagBuf;
            }               
            break;
            
         case 2:
            if (ch == '\n')
               x = 3;
            else
            {
               *(cp++) = '\\';
               *cp = '\0';
               if (strlen(valBuf) >= sizeof(valBuf) - 1)
                  rv = 0;
               x = 5;
            }
            break;
         
         case 3:
            if (isspace(ch) && ch != '\n')
               x = 3;
            else
            {
               *(cp++) = ' ';
               *cp = '\0';
               if (strlen(valBuf) >= sizeof(valBuf) - 1)
                  rv = 0;
               x = 5;
            }
            break;
         
         case 4:
            if (isspace(ch) && ch != '\n')
               x = 4;
            else
               x = 5;
            break;
            
         default:
            rv = 0;
      }
      if (dbg > 2) fprintf(stdout, "xDefaults::load(): - x = %d, ch = 0x%x\n", x, ch);
      if (dbg) fflush(stdout);
   }
   return(rv);
}

void xDefaults::show()
{
   xDefltEntry *pEntry;
   
   for (pEntry = pDefaults; pEntry != NULL; pEntry = pEntry->next())
      fprintf(stdout, "Default: %s=|%s|\n", (const char *)pEntry->tag, (const char *)pEntry->value);
}

void xDefaults::setEscapes(const char escapes[][2])
{
   int x;

   if (dbg > 0) fprintf(stdout, "xDefaults::setEscapes():Enter\n");
   for (x = 0; escapes[x][0] != 0; x++)
   {
      if (dbg > 0) fprintf(stdout, "xDefaults::setEscapes():char = 0x%x, replacement = 0x%x\n",
                                    escapes[x][0], escapes[x][1]);
      escTable[escapes[x][0]] = escapes[x][1];
   }
}      
