/***************************************************************************
**    xDefaults.h  $Revision: 1.3 $ - $Name:  $ 
**    Reads Defaults file
**
**    Copyright (C) 1996 Joseph Croft <jcroft@unicomp.net>
**
**    This library is free software; you can redistribute it and/or
**    modify it under the terms of the GNU Library General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    This library is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**    Library General Public License for more details.
**
**    You should have received a copy of the GNU Library General Public
**    License along with this library; if not, write to the Free
**    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#ifndef _xDDEFAULTS_H
#define _xDDEFAULTS_H

#include <stdio.h>
#include <qstring.h>

class xDefaults;
class xDefltEntry;

class xDefltEntry
{
public:
   xDefltEntry(const char *pTag, const char *pValue);
   ~xDefltEntry();
   
   bool is(const char *pTag);
   const char *val(const char *pVal);
   const char *val(xDefltEntry *pDefaults, bool expand = TRUE);

   friend class xDefaults;
   
private:
   void strReplaceEnvArgs(xDefltEntry *pDefaults, QString &value,
                          QString &buf);
   xDefltEntry *next() { return(pNext); };
   xDefltEntry *prev() { return(pPrev); };
   
   void link(xDefltEntry *pEntry)
   {
      pNext = pEntry->setNext(this);
      if (pNext)
         pNext->setPrev(this);
      pPrev = pEntry;
   };
   
   void unlink()
   {
      if (pNext)
         pNext->setPrev(pPrev);
      if (pPrev)
         pPrev->setNext(pNext);
   };
   
   xDefltEntry *setNext(xDefltEntry *pEntry) 
   {
      xDefltEntry *b;
      b = pNext;
      pNext = pEntry;
      return(b);
   };
   
   xDefltEntry *setPrev(xDefltEntry *pEntry)
   {
      xDefltEntry *b;
      b = pPrev;
      pPrev = pEntry;
      return(b);
   };

   xDefltEntry *pNext;
   xDefltEntry *pPrev;
   QString  tag;
   QString  value;
   QString  expanded;
};

class xDefaults
{
public:
   xDefaults();
   ~xDefaults();
   
   xDefltEntry *add(const char *pTag, const char *pVvalue);
   const char *get(const char *pTag, bool expand = TRUE);
   const char *set(const char *pTag, const char *pValue);
   void setEscapes(const char escapes[][2]);
   bool load(FILE *pFile, char *pDefaults[]);
   void show();

private:
   xDefltEntry *find(const char *pTag);
   xDefltEntry *pDefaults;
};

#endif
