/***************************************************************************
**    xDialog.cpp  $Revision: 1.6 $ - $Name:  $ 
**    Self fitting Dialog and Frame classes
**
**    Copyright (C) 1996 Joseph Croft <jcroft@unicomp.net>
**
**    This library is free software; you can redistribute it and/or
**    modify it under the terms of the GNU Library General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    This library is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**    Library General Public License for more details.
**
**    You should have received a copy of the GNU Library General Public
**    License along with this library; if not, write to the Free
**    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include <stdio.h>
#include <qsize.h>
#include "xDialog.h"

static int dbg = 0;

xFrame::xFrame(QWidget *pParent, const char *pName) :
                  QFrame(pParent, pName)
{
   if (dbg) fprintf(stdout, "xFrame::xFrame():Enter\n");
   
   fc = 0;
   initMetaObject();
   widgetAlgn = xALIGN_Vert;
   widgetResz = xSPACE_ToFit;
   widgetSpac = 10;
   widgetVMarg = 10;
   widgetHMarg = 10;
   
   widgetListLength = 10;
   widgetCount = 0;
   fitFrameRecurse = 0;
   pWidgetList = new QWidget*[10];
   if (dbg) fprintf(stdout, "xFrame::xFrame():Exit\n");
}

xFrame::xFrame(int align, int resz, int space, QWidget *pParent, const char *pName) :
                  QFrame(pParent, pName)
{
   if (dbg) fprintf(stdout, "xFrame::xFrame():Enter\n");
   initMetaObject();
   widgetAlgn = align;
   widgetResz = resz;
   widgetSpac = space;
   widgetVMarg = 10;
   widgetHMarg = 10;
   
   widgetListLength = 10;
   widgetCount = 0;
   pWidgetList = new QWidget*[10];
   if (dbg) fprintf(stdout, "xFrame::xFrame():Exit\n");
}

xFrame::xFrame()
{
   if (pWidgetList)
      delete [] pWidgetList;
}

int xFrame::addWidget(QWidget *pWidget, QWidget *pNeighbor, int relativePos)
{
   QWidget *pTmpWidget, **pNewList;
   int iOffset, rv, x = 0;
   
   if (dbg) fprintf(stdout, "xFrame::addWidget():Enter, pWidget = %d\n", (int)pWidget);
   if (dbg) fflush(stdout);
   rv = 0;
   
   if (widgetCount == 0)
   {
      widgetCount++;
      pWidgetList[0] = pWidget;
      if (dbg) fprintf(stdout, "xFrame::addWidget():Added widget: %d at position 0\n", (int)pWidgetList[0]);
      if (dbg) fflush(stdout);
      if (dbg) fprintf(stdout, "xFrame::addWidget():exit(%d)\n", rv);
      if (dbg) fflush(stdout);
      return(rv);
   }
     
   if (widgetCount + 1 >= widgetListLength)
   {
      if (dbg) fprintf(stdout, "xFrame::addWidget():Adding 10 more Widget entries to the array\n");
      if (dbg) fflush(stdout);
      widgetListLength += 10;
      pNewList = new QWidget*[widgetListLength];
      for (x = 0; x < widgetCount; x++)
         pNewList[x] = pWidgetList[x];
      delete [] pWidgetList;
      pWidgetList = pNewList;
   }
   
   iOffset = widgetCount;
   if (pNeighbor)
   {
      if (dbg) fprintf(stdout, "xFrame::addWidget():Searching for neighbor\n");
      if (dbg) fflush(stdout);
      for (pTmpWidget = NULL, iOffset = 0; x < widgetCount; x++)
         if (pWidgetList[x] == pNeighbor)
            break;
   }
   else if (relativePos == xPOS_Left && widgetCount > 0)
      iOffset = widgetCount - 1;
      
   if (iOffset < widgetCount)
   {
      if (dbg) fprintf(stdout, "xFrame::addWidget():Inserting Widget at pos %d\n", iOffset);
      if (dbg) fflush(stdout);
      widgetCount++;
      if (relativePos == xPOS_Left)
      {
         for (x = widgetCount; x >= iOffset; x--)
            pWidgetList[x] = pWidgetList[x - 1];
         pWidgetList[iOffset] = pWidget;
      }
      else
      {
         if (iOffset < widgetCount - 2)
         {
            for (x = widgetCount; x > iOffset; x--)
               pWidgetList[x] = pWidgetList[x - 1];
            pWidgetList[iOffset + 1] = pWidget;
         }
         else
            pWidgetList[widgetCount - 1] = pWidget;
      }
   }
   else
   {
      pWidgetList[iOffset] = pWidget;
      widgetCount++;
      if (dbg) fprintf(stdout, "xFrame::addWidget():Added widget: %d at position %d\n", (int)pWidgetList[iOffset], iOffset);
      if (dbg) fflush(stdout);
   }
      
   if (dbg) fprintf(stdout, "xFrame::addWidget():exit(%d)\n", rv);
   if (dbg) fflush(stdout);
   return(rv);
}

void xFrame::resizeEvent(QResizeEvent *pEvt)
{
   QSize sizeTmp;
   
   sizeTmp = pEvt->size();
   fitFrame(&sizeTmp);
   QFrame::resizeEvent(pEvt);
}

void xFrame::resize(QSize *frameSize)
{
   if (dbg) fprintf(stdout, "xFrame::resize(QSize*):frameSize.w = %d, .h = %d\n",
                    frameSize->width(), frameSize->height());
   if (dbg) fflush(stdout);
   fitFrame(frameSize);
   if (dbg) fprintf(stdout, "xFrame::resize(QSize*):Calling QWidget::resize()\n");
   if (dbg) fflush(stdout);
   QWidget::resize(*frameSize);
   if (dbg) fprintf(stdout, "xFrame::resize(QSize*):Exit\n");
   if (dbg) fflush(stdout);
}

void xFrame::fitFrame(const QSize *frameSize)
{
   int x, maxSize, offset, totalSize, currentSize, x1, x2, x3, x4, x5;
   int frameH = 0, frameW = 0, offsetV, offsetH;

   if (fc > 1)
      return;
   fc++;
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Enter\n", fc, className());
   if (dbg) fflush(stdout);
   /*
   if (fitFrameRecurse > 0)
   {
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Exit\n", fc, className());
      if (dbg) fflush(stdout);
      fc--;
      return;
   }
   else
      fitFrameRecurse++;
   */      
   move(0, 0);
   
   if (frameSize != NULL)
   {
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):frameSize.w = %d, .h = %d\n", fc, className(),
                       frameSize->width(), frameSize->height());
      if (dbg) fflush(stdout);
   }
   
   //
   // Start by fitting any child frames that have been added
   // to this frame. Always allow the children to size themselves.
   // 
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Frame Name: |%s|\n", fc, className(), name());
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Frame Class: |%s|\n", fc, className(), className());
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Parents Frame Name: |%s|\n", fc, className(), parent()->name());
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Parents Frame Class: |%s|\n", fc, className(), parent()->className());
   if (dbg) fflush(stdout);
   for (x = 0; x < widgetCount; x++)
   {
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Testing to fit child frame %d\n", fc, className(), x);
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Frame %d Name: |%s|\n", fc, className(), x, pWidgetList[x]->name());
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Frame %d Class: |%s|\n", fc, className(), x, pWidgetList[x]->className());
      if (dbg) fflush(stdout);
      if (pWidgetList[x]->inherits("xFrame"))
      {
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Widget %d is child frame\n", fc, className(), x);
         if (dbg) fflush(stdout);
         ((xFrame *)pWidgetList[x])->fitFrame();
      }
   }
      
   //
   // Now see who's the largest of the bunch. This will keep them 
   // from looking jagged as they go down or across
   //
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Finding largest frame\n", fc, className());
   if (dbg) fflush(stdout);
   for (maxSize = 0, x = 0; x < widgetCount; x++)
   {
      if (widgetAlgn == xALIGN_Vert)
         maxSize = (maxSize > pWidgetList[x]->size().width()) ? maxSize :
                                                                pWidgetList[x]->size().width();
      else
         maxSize = (maxSize > pWidgetList[x]->size().height()) ? maxSize :
                                                                 pWidgetList[x]->size().height();
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Widget[%d].w = %d .h = %d\n", fc, className(), x,
                                pWidgetList[x]->size().width(),
                                pWidgetList[x]->size().height());
   }
   
   //
   // Now make every one the same size as the largest Object
   // If we resize a xFrame then fit it also
   //
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):sizing frames\n", fc, className());
   if (dbg) fflush(stdout);
   for (x = 0; x < widgetCount; x++)
   {
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Resizing Widget %d\n", fc, className(), x);
      if (widgetAlgn == xALIGN_Vert)
         pWidgetList[x]->QWidget::resize(maxSize, pWidgetList[x]->size().height());
      else
         pWidgetList[x]->QWidget::resize(pWidgetList[x]->size().width(), maxSize);
         
      if (pWidgetList[x]->inherits("xFrame"))
      {
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Widget %d is child frame\n", fc, className(), x);
         if (dbg) fflush(stdout);
         ((xFrame *)pWidgetList[x])->fitFrame(&pWidgetList[x]->size());
      }
   }
   
   //
   // Okay hear comes the hard part! Fit them into the window
   // Using the constraints given!
   //
   // Start by getting the total width (height) needed by all the
   // Widgets in the frame.
   //
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Tabulating required width/height\n", fc, className());
   if (dbg) fflush(stdout);
   for (totalSize = 0, x = 0; x < widgetCount; x++)
   {
      if (widgetAlgn == xALIGN_Vert)
         totalSize += pWidgetList[x]->size().height();
      else
         totalSize += pWidgetList[x]->size().width();
   }

   //
   // Okay, if we can resize the frame, then we will use the given
   // parameters. Otherwise, fit them in to suit, if the frame is too
   // small just ignore the request!
   //
   ///////////////////////////////////////////////////////////////////////
   //
   // Okay, position the Widgets starting at the top (vert) or left (horz)
   // margins spacing them as requested.
   //
   offsetH = frameWidth() + widgetHMarg;
   offsetV = frameWidth() + widgetVMarg;
   
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Adjusting widget positions\n", fc, className());
   if (dbg) fflush(stdout);
   if (frameSize == NULL)
   {
      for (x = 0; x < widgetCount; x++)
      {
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):moving widget %d to h=%d, v=%d\n", fc, className(), x, offsetH, offsetV);
         if (dbg) fflush(stdout);
         pWidgetList[x]->move(offsetH, offsetV);
            
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):calculating offset for next widget\n", fc, className());
         if (dbg) fflush(stdout);
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Widget size: h=%d X w=%d\n", fc, className(),
                          pWidgetList[x]->size().height(), pWidgetList[x]->size().width());
         if (dbg) fflush(stdout);
         if (widgetAlgn == xALIGN_Vert)
            offsetV += pWidgetList[x]->size().height();
         else
            offsetH += pWidgetList[x]->size().width();
            
         if (x < widgetCount - 1)
         {
            if (widgetAlgn == xALIGN_Vert)
               offsetV += widgetSpac;
            else
               offsetH += widgetSpac;
         }
      }
      offset = (widgetAlgn == xALIGN_Vert) ? offsetV + widgetVMarg :
                                                  offsetH + widgetHMarg;
      
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Total width/height = %d\n", fc, className(), offset);
      if (dbg) fflush(stdout);
      
      //
      // Now resize the frame to fit snuggly around the Widgets, also set the
      // minimum size of the frame to the size calculated
      //
      if (widgetAlgn == xALIGN_Vert)
      {
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Setting frame size to: %d X %d\n", fc, className(),
                          pWidgetList[0]->size().width() + (widgetHMarg * 2), offset);
         if (dbg) fflush(stdout);
         frameH = offset + (frameWidth() * 2);
         frameW = pWidgetList[0]->size().width() + (widgetHMarg * 2) +
                  (frameWidth() * 2);
      }
      else
      {
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Setting frame size to: %d X %d\n", fc, className(),
                          offset, pWidgetList[0]->size().height() + (widgetVMarg * 2));
         if (dbg) fflush(stdout);
         frameW = offset + (frameWidth() * 2);
         frameH = pWidgetList[0]->size().height() + (widgetVMarg * 2) +
                  (frameWidth() * 2);
      }
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Setting frame size to: w=%d X h=%d\n", fc, className(),
                       frameW, frameH);
      if (dbg) fflush(stdout);
      QWidget::resize(frameW, frameH);
//    setMinimumSize(frameW, frameH);
   }
   
   //
   // Okay, the frame is being resized! The default is to position the
   // widgets in the frame to fit within the given margins. Otherwise, if 
   // specified, the widgets will be resized to fit evenly across the
   // frame preserving the spacing and margins as best we can.
   //
   else
   {
      //
      // Here we calculate the distance to seperate the Widgets to
      // evenly space them across (or down) the frame, whose 
      // size is specified by frameSize, taking into account
      // the Margins set by wigetVMarg and widgetHMarg.
      //
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Frame is being resized\n", fc, className());
      if (dbg) fflush(stdout);
      
      currentSize = (widgetAlgn == xALIGN_Vert) ? frameSize->height() :
                                                   frameSize->width();
      x3 = 2 * ((widgetAlgn == xALIGN_Vert) ? widgetVMarg : widgetHMarg);
      x4 = 2 * frameWidth();
      x1 = currentSize - (totalSize + x3 + x4);
      if (widgetResz == xSPACE_ToFit)
         x2 = x1 / (widgetCount + 1);
      else
         x2 = 0;
      
      if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):currentSize = %d, totalSize = %d, x1 = %d, x2 = %d, x3 = %d, x4 = %d\n", fc, className(),
                                currentSize, totalSize, x1, x2, x3, x4);
      
      if (widgetResz != xSPACE_ToFit)
      {
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Sizing Widgets to fit\n", fc, className());
         currentSize = (widgetAlgn == xALIGN_Vert) ? frameSize->width() :
                                                      frameSize->height();
         x3 = 2 * ((widgetAlgn == xALIGN_Vert) ? widgetHMarg : widgetVMarg);
         x4 = 2 * frameWidth();
         x5 = (widgetAlgn == xALIGN_Vert) ? pWidgetList[0]->size().width() :
                                             pWidgetList[0]->size().height();
         x1 = currentSize - (x5 + x3 + x4);
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):currentSize = %d, totalSize = %d, x1 = %d, x3 = %d, x4 = %d, x5 = %d\n", fc, className(),
                                   currentSize, totalSize, x1, x3, x4, x5);
      }
      
      offset = frameWidth() + x2;
      offset += (widgetAlgn == xALIGN_Vert) ? widgetVMarg : widgetHMarg;
      for (x = 0; x < widgetCount; x++)
      {
         if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Offset to widget %d = %d\n", fc, className(),
                                   x, offset);
         
         if (widgetResz != xSPACE_ToFit)
         {
            if (widgetAlgn == xALIGN_Vert)
            {
               pWidgetList[x]->resize(pWidgetList[x]->size().width() + x1,
                                      pWidgetList[x]->size().height());
            }
            else
            {
               pWidgetList[x]->resize(pWidgetList[x]->size().width(),
                                      pWidgetList[x]->size().height() + x1);
            }
            if (pWidgetList[x]->inherits("xFrame"))
            {
               if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Widget %d is child frame\n", fc, className(), x);
               if (dbg) fflush(stdout);
               ((xFrame *)pWidgetList[x])->fitFrame();
            }
         }
         
         if (widgetAlgn == xALIGN_Vert)
            pWidgetList[x]->move(widgetHMarg, offset);
         else
            pWidgetList[x]->move(offset, widgetVMarg);
            
         offset += (widgetAlgn == xALIGN_Vert) ?
            pWidgetList[x]->size().height() : pWidgetList[x]->size().width();
            
         if (widgetResz != xSPACE_Resize)
            offset += x2;
         else
            offset += widgetSpac;
      }
      // QWidget::resize(frameSize->width(), frameSize->height());
   }
   fitFrameRecurse--;
   if (dbg) fprintf(stdout, "xFrame::fitFram.%d(%s):Exit\n", fc, className());
   if (dbg) fflush(stdout);
   fc--;
}

void xFrame::removeWidget(QWidget *pWidget)
{
   int iOffset, x;
   
   for (x = 0; x < widgetCount; x++)
      if (pWidget == pWidgetList[x])
         break;
         
   if (x < widgetCount)
   {
      iOffset = x;
      widgetCount--;
      for (x = widgetCount; x > iOffset; x--)
         pWidgetList[x - 1] = pWidgetList[x];
   }
}

void xFrame::setAlignment(int alignment)
{
   widgetAlgn = alignment;
}

void xFrame::setWidgetSpacing(int spacing)
{
   widgetSpac = spacing;
}

void xFrame::setMargins(int vertMargin, int horzMargin)
{
   widgetVMarg = vertMargin;
   widgetHMarg = horzMargin;
}

void xFrame::margins(int& vertMargin, int& horzMargin)
{
   vertMargin = widgetVMarg;
   horzMargin = widgetHMarg;
}

void xFrame::doubleClickEvent(QMouseEvent *pEvt)
{
   int x;

   if (dbg) fprintf(stdout, "xFrame::doubleClickEvent(%s):Have Double Click!\n", className());
   for (x = 0; x < widgetCount; x++)
   {
      if (pWidgetList[x]->inherits("QWidget"))
      {
         if (pWidgetList[x]->rect().contains(pEvt->pos(), TRUE))
         {
            if (dbg) fprintf(stdout, "xFrame::doubleClickEvent(%s):Sending Event to child: %d:%s\n",
                                     className(), x, pWidgetList[x]->className());
//            pWidgetList[x]->mouseDoubleClickEvent(pEvt);
         }
      }
   }
   if (dbg) fprintf(stdout, "xFrame::doubleClickEvent(%s):Exit\n", className());
}

xDialog::xDialog(QWidget *pParent, const char *pName, bool modal) :
            QDialog(pParent, pName, modal)
{
   if (dbg) fprintf(stdout, "xDialog::xDialog():Enter\n");
   needsDeleting = TRUE;
   mainFrame = new xFrame(this);
   if (mainFrame)
      QDialog::resize(mainFrame->size());
   if (dbg) fprintf(stdout, "xDialog::xDialog():Exit\n");
}

xDialog::xDialog(xFrame *pFrame, QWidget *pParent,
                   const char *pName)
{
   // Appease the Compiler Warnings
   pParent = pParent;
   pName = pName;

   if (pFrame)
   {
      needsDeleting = FALSE;
      mainFrame = pFrame;
   }
   else
   {
      needsDeleting = TRUE;
      mainFrame = new xFrame(this);
   }
   if (mainFrame)
      QDialog::resize(mainFrame->size());
}

xDialog::~xDialog()
{
   if (needsDeleting && mainFrame)
      delete mainFrame;
}

int xDialog::addWidget(QWidget *pWidget, QWidget *pNeighbor, int relativePos)
{
   return(mainFrame->addWidget(pWidget, pNeighbor, relativePos));
}
   
void xDialog::removeWidget(QWidget *pWidget)
{
   mainFrame->removeWidget(pWidget);
}

void xDialog::setAlignment(int alignment)
{
   mainFrame->setAlignment(alignment);
}

void xDialog::setWidgetSpacing(int space)
{
   mainFrame->setWidgetSpacing(space);
}

void xDialog::setMargins(int vertMargin, int horzMargin)
{
   mainFrame->setMargins(vertMargin, horzMargin);
}
   
void xDialog::margins(int& vertMargin, int& horzMargin)
{
   int vm, hm;
   mainFrame->margins(vm, hm);
   vertMargin = vm; horzMargin = hm;
}

void xDialog::setResizeMode(int reszMode)
{
   mainFrame->setResizeMode(reszMode);
}

void xDialog::fitFrame(const QSize *frameSize)
{
   if (mainFrame)
   {
      mainFrame->fitFrame(frameSize);
//      QDialog::resize(mainFrame->size());
   } 
}

void xDialog::initFrame(const QSize *frameSize)
{
   if (dbg) fprintf(stdout, "xDialog::initFrame():Enter\n");
   if (mainFrame)
   {
      mainFrame->fitFrame(frameSize);
      QWindow::resize(mainFrame->size());
   } 
   if (dbg) fprintf(stdout, "xDialog::initFrame():Exit\n");
}
   
void xDialog::resize(QSize *frameSize)
{
     mainFrame->QWidget::resize(frameSize->width(), frameSize->height());
}

void xDialog::resizeEvent(QResizeEvent *pEvt)
{
   if (resizeRecurse < 1)
   {
      resizeRecurse++;
      fitFrame(&(pEvt->size()));
      resizeRecurse--;
   }
   QDialog::resizeEvent(pEvt);
}

void xDialog::doubleClickEvent(QMouseEvent *pEvt)
{
   if (mainFrame)
      mainFrame->doubleClickEvent(pEvt);
}

#include "xDialog.moc"

