/***************************************************************************
**    xDialog.h  $Revision: 1.4 $ - $Name:  $ 
**    Self fitting Dialog and Frame classes
**
**    Copyright (C) 1996 Joseph Croft <jcroft@unicomp.net>
**
**    This library is free software; you can redistribute it and/or
**    modify it under the terms of the GNU Library General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    This library is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**    Library General Public License for more details.
**
**    You should have received a copy of the GNU Library General Public
**    License along with this library; if not, write to the Free
**    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#ifndef _xDIALOG_H
#define _xDIALOG_H

#include <qdialog.h>
#include <qframe.h>
#include <qwindow.h>
#include <qwidget.h>

#define  xALIGN_Vert         0
#define  xALIGN_Horz         1

#define  xSPACE_ToFit        0
#define  xSPACE_Resize       1

#define  xPOS_Right          0
#define  xPOS_Left           1
#define  xPOS_Below          0
#define  xPOS_Above          1

class xDialog;

class xFrame : public QFrame
{
   Q_OBJECT

   friend class xDialog;
      
public:
   xFrame(QWidget *pParent = NULL, const char *pName = NULL);
   xFrame(int align = xALIGN_Vert, int resz = xSPACE_ToFit,
                 int space = 10, QWidget *pParent = NULL,
                  const char *pName = NULL);
   xFrame();
   
   int addWidget(QWidget *pWidget, QWidget *pNeighbor = NULL,
             int relativePos = xPOS_Right);
   void removeWidget(QWidget *pWidget);
   void setAlignment(int alignment = xALIGN_Vert);
   void setWidgetSpacing(int space = 0);
   void setMargins(int vertMargin = 10, int horzMargin = 10);
   void setResizeMode(int reszMode) { widgetResz = reszMode; };
   void margins(int& vertMargin, int& horzMargin);
   int widgetSpacing() { return(widgetSpac); };
   virtual void fitFrame(const QSize *frameSize = NULL);
   virtual void resize(QSize *frameSize);
   int resizeMode() { return(widgetResz); };
   
private:
   int      widgetAlgn;
   int      widgetSpac;
   int      widgetListLength;
   int      widgetCount;
   int      widgetVMarg;
   int      widgetHMarg;
   int      widgetResz;
   int      fitFrameRecurse;
   int      fc;
   QWidget  **pWidgetList;

protected:
   void resizeEvent(QResizeEvent *pEvt);
   virtual void doubleClickEvent(QMouseEvent *pEvt);
};

class xDialog : public QDialog
{
public:
   xDialog(QWidget *pParent = NULL, const char *pName = NULL,
            bool modal = FALSE);
   xDialog(xFrame *pFrame = NULL, QWidget *pParent = NULL, 
            const char *pName = NULL);
   ~xDialog();
   
   xFrame  *mainFrame;

private:
   bool     needsDeleting;
   int      resizeRecurse;

public:   
   int addWidget(QWidget *pWidget, QWidget *pNeighbor = NULL, int relativePos = xPOS_Right);
   void removeWidget(QWidget *pWidget);
   void setAlignment(int alignment = xALIGN_Vert);
   void setWidgetSpacing(int space = 0);
   void setMargins(int vertMargin = 10, int horzMargin = 10);
   void margins(int& vertMargin, int& horzMargin);
   void setResizeMode(int reszMode);
   int resizeMode() { return(mainFrame->resizeMode()); };
   int widgetSpacing() { return(mainFrame->widgetSpacing()); };
   void fitFrame(const QSize *frameSize = NULL);
   void initFrame(const QSize *frameSize = NULL);
   
protected:
   virtual void resize(QSize *frameSize);
   virtual void resizeEvent(QResizeEvent *pEvt);
   virtual void doubleClickEvent(QMouseEvent *pEvt);
};

#endif

