/***************************************************************************
**    xEdit.cpp  $Revision: 1.3 $ - $Name:  $ 
**    QEdit w/ Macro Replacement
**
**    Copyright (C) 1996 Joseph Croft <jcroft@unicomp.net>
**
**    This library is free software; you can redistribute it and/or
**    modify it under the terms of the GNU Library General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    This library is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**    Library General Public License for more details.
**
**    You should have received a copy of the GNU Library General Public
**    License along with this library; if not, write to the Free
**    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include <ctype.h>
#include "xDefaults.h"
#include "xEdit.h"
#include <qlined.h>

extern xDefaults Defaults;

static int dbg = 0;

xEdit::xEdit(QWidget *pParent, const char *pName) :
        QLineEdit(pParent, pName)
{
   txtStr = NULL;
   setAcceptFocus(TRUE);
}

QString xEdit::macroReplace(const char *cp)
{
   QString rv, strTmp1, strTmp2;
   char ch, ch1, ch2;
   int base;

   if (dbg) fprintf(stdout, "xEdit::macroReplace():Enter\n");
   if (cp == NULL)
   {
      rv = "";
      return(rv);
   }
   for (ch = ' ', rv = ""; *cp;)
   {
      if (ch != '\\' && ch != '$' && *cp == '$')
      {
         if (*(cp + 1) != '$')
         {
            if (dbg) fprintf(stdout, "xEdit::macroReplace():Test for Macro in: |%s|\n", cp);
            for (strTmp1 = "", cp++; *cp && isalnum(*cp) || *cp == '.' || *cp == '_'; cp++)
               strTmp1 += *cp;
            if (dbg) fprintf(stdout, "xEdit::macroReplace():Tag =  |%s|\n", (const char *)strTmp1);
            if ((strTmp2 = macroReplace(Defaults.get(strTmp1))) != NULL)
            {
               if (dbg) fprintf(stdout, "xEdit::macroReplace():Found Entry!! = |%s|\n", (const char *)strTmp1);
               rv += strTmp2;
            }
            if (dbg) fprintf(stdout, "xEdit::macroReplace():Remaining string to test = |%s|\n", cp);
         }
         else
            ch = *(cp++);
      }
      else if (ch != '\\' && *cp == '\\')
      {
         if (dbg) fprintf(stdout, "xEdit::macroReplace():Found escape char!\n");
         ch = *(cp++);
      }
      else if (ch == '\\')
      {
         if (dbg) fprintf(stdout, "xEdit::macroReplace():Interpreting escape char!\n");
         switch (toupper(*cp))
         {
            case 'B':
               if (dbg) fprintf(stdout, "xEdit::macroReplace():Found bold char!\n");
               ch = '\x02';
               rv += ch;
               cp++;
               break;

            case 'U':
               if (dbg) fprintf(stdout, "xEdit::macroReplace():Found underline char!\n");
               ch = '_' - 0x40;
               rv += ch;
               cp++;
               break;

            case 'X':
            case '0':
            case '1':
            case '2':
            case '3':
            case '4':
            case '5':
            case '6':
            case '7':
            case '8':
            case '9':
               if (dbg) fprintf(stdout, "xEdit::macroReplace():Found Number!!!!!\n");
               if (toupper(*cp) == 'X')
               {
                  base = 16;
                  cp++;
               }
               else if (*cp == '0')
                  base = 8;
               else
                  base = 10;
               for (ch1 = 0; base != 0;)
               {
                  if (base == 16)
                  {
                     if (isxdigit(*cp))
                     {
                        ch1 *= 16;
                        if ((ch2 = toupper(*cp) - '0') > 9)
                           ch1 += ch2 - 7;
                        else
                           ch1 += ch2;
                        cp++;
                     }
                     else
                        base = 0;
                  }
                  else if (base == 8)
                  {
                     if (dbg) printf("ctoi():base 8, ch = 0x%x, %c\n\r", ch, ch);
                     if (*cp >= '0' && *cp <= '7')
                     {
                        ch1 *= 8;
                        ch1 += *cp - '0';
                        cp++;
                        if (dbg) printf("ctoi():base 8, rslt = 0%x\n\r", ch1);
                     }
                     else
                        base = 0;
                  }
                  else
                  {
                     if (dbg) printf("ctoi():base 10, *cp = 0x%x, %c\n\r", *cp, *cp);
                     if (isdigit(*cp))
                     {
                        ch1 *= 10;
                        ch1 += *cp - '0';
                        cp++;
                        if (dbg) printf("ctoi():base 10, rslt = 0x%x\n\r", ch1);
                     }
                     else
                        base = 0;
                  }
               }
               ch = ch1;
               rv += ch1;
               break;

            default:
               if (dbg) fprintf(stdout, "xEdit::macroReplace():Found defaults!!!!!!!!\n");
               ch = ' ';
               rv += *(cp++);
               break;
         }
      }
      else
      {
         ch = *cp;
         rv += *(cp++);
      }
   }
   if (dbg) fprintf(stdout, "xEdit::macroReplace():Exit |%s|\n", (const char *)rv);
   return(rv);
}

const char *xEdit::text()
{
   const char *cp;

   if (dbg) fprintf(stdout, "xEdit::text():Enter\n");
   if (txtStr != NULL)
      delete txtStr;
   cp = QLineEdit::text();
   if (dbg) fprintf(stdout, "xEdit::text():Intermediate cp = |%s|\n", cp);
   txtStr = new QString(macroReplace(cp));
   if (dbg) fprintf(stdout, "xEdit::text():Exit |%s|\n", (const char *)*txtStr);
   return((const char *)*txtStr);
}
