/***************************************************************************
**    xSocket.cpp  $Revision: 1.7 $ - $Name:  $ 
**    Class to manipulate sockets
**
**    Copyright (C) 1996 Joseph Croft <jcroft@unicomp.net>
**
**    This library is free software; you can redistribute it and/or
**    modify it under the terms of the GNU Library General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    This library is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**    Library General Public License for more details.
**
**    You should have received a copy of the GNU Library General Public
**    License along with this library; if not, write to the Free
**    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <setjmp.h>
#include <qmsgbox.h>
#include "xSocket.h"

#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff
#endif

extern int errno;
extern char *sys_errlist[];

static int dbg = 0;
static bool timedOut = FALSE, retryFlag = FALSE;
static volatile bool unfinished;

// static jmp_buf timeout;

xSocketTCP::xSocketTCP(QObject *pParent, char *pName) :
            QObject(pParent, pName)
{
   connectFlag = FALSE;
   fd = -1;
   dbf = fopen("qtTextOut.txt", "w");
   snw = NULL;
   snr = NULL;
   sna = NULL;
   snc = NULL;
   pTimer = new QTimer(this);
}   

xSocketTCP::~xSocketTCP()
{
   if (snw)
      delete snw;
   snw = NULL;
   if (snr)
      delete snr;
   snr = NULL;
   if (dbf)
      fclose(dbf);
   if (pTimer)
      delete pTimer;
}

void xSocketTCP::readyRead()
{
   int n;
   char buf[512];

   if ((n = ::read(fd, buf, sizeof(buf - 1))) < 0)
   {
      close(fd);
      if (snw)
         delete snw;
      snw = NULL;
      if (snr)
         delete snr;
      snr = NULL;
      connectFlag = FALSE;
      emit socketError(ESOCK_BadRead);
      emit socketClosed();
   }
   else if (n > 0)
   {
      if (dbg > 3) fprintf(stdout, "xSocketTCP::readyRead():Read %d chars from socket\n", n);
      if (dbg) fflush(stdout);
      buf[n] = '\0';
      if (dbf) fprintf(dbf, buf);
      if (dbf) fflush(dbf);
      if (dbg > 3) fprintf(stdout, "xSocketTCP::readyRead():Emitin readFromSocket signal\n");
      if (dbg > 3) fflush(stdout);
      emit readFromSocket(buf);
      if (dbg > 3) fprintf(stdout, "xSocketTCP::readyRead():Back from sending signal\n");
      if (dbg > 3) fflush(stdout);
   }
   else
   {
      close(fd);
      fd = -1;
      connectFlag = FALSE;
      if (dbg) fprintf(stdout, "xSocketTCP::readyRead():SOCKET CLOSED!\n");
      if (dbg) fflush(stdout);
      if (snw)
         delete snw;
      snw = NULL;
      if (snr)
         delete snr;
      snr = NULL;
      emit socketClosed();
   }
}

void xSocketTCP::readyWrite()
{
   int n;
   char buf[512];

   if (dbg) fprintf(stdout, "xSocketTCP::readyWrite():Enter\n");
   if (dbg) fflush(stdout);
   if ((n = writeBuffer.length()) > 0)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::readyWrite():writing %d chars, |%s|, to socket\n", n, (const char *)writeBuffer);
      if (dbg) fflush(stdout);
      sprintf(buf, "Client>>|%s|\n", (const char *)writeBuffer);
      if (dbf) fprintf(dbf, buf);
      if (dbf) fflush(dbf);
      if ((n = ::write(fd, (const char *)writeBuffer, n)) > 0)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::readyWrite():Wrote %d chars to socket\n", n);
         if (dbg) fprintf(stdout, "xSocketTCP::readyWrite():Removing string from buffer\n");
         if (dbg) fflush(stdout);
         writeBuffer.remove(0, n);

         if (dbg) fprintf(stdout, "xSocketTCP::readyWrite():Chars removed\n");
         if (dbg) fflush(stdout);
      }
      else if (n < 0)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::readyWrite():Error writing!\n");
         if (dbg) fflush(stdout);
         writeBuffer = "";
         close(fd);
         connectFlag = FALSE;
         emit socketError(ESOCK_BadWrite);
         if (snw)
            delete snw;
         snw = NULL;
         if (snr)
            delete snr;
         snr = NULL;
      }
      else
      {
         if (dbg) fprintf(stdout, "xSocketTCP::readyWrite():Write SOCKET CLOSED!\n");
         if (dbg) fflush(stdout);
         close(fd);
         fd = -1;
         writeBuffer = "";
         connectFlag = FALSE;
         emit socketClosed();
         if (snw)
            delete snw;
         snw = NULL;
         if (snr)
            delete snr;
         snr = NULL;
      }
      if (snw)
         snw->setEnabled(!writeBuffer.isEmpty());
      if (dbg) fprintf(stdout, "xSocketTCP::readyWrite():Done\n");
      if (dbg) fflush(stdout);
   }
}

void xSocketTCP::readyAccept(int sock)
{
   int n;

   if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Enter, sock = %d\n", sock);
   if (dbg) fflush(stdout);
   n = sizeof(acceptAddr);
   if ((fd = ::accept(sock, (struct sockaddr *)&acceptAddr, &n)) < 0)
/*
   {
      sprintf(errBuf, "Error Accpeting Socket Connection:%s", strerror(errno));
      QMessageBox::message("Error", errBuf);
      connectFlag = FALSE;
      emit connFailed(errno);
      close(sock);
   }
*/
   {
      if (errno != EWOULDBLOCK &&
          errno != EINPROGRESS &&
          errno != EALREADY &&
          errno != EAGAIN)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Error Found!\n");
         if (dbg) fflush(stdout);
         close(fd);
         perror("xSocketTCP::readyAccept():Connection Failed!!!\n");
         if (sna)
         {
            if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Disconnecting socket!\n");
            if (dbg) fflush(stdout);
            QObject::disconnect(sna, SIGNAL(activated(int)), this, SLOT(readyAccept(int)));
            delete sna;
         }
         sna = NULL;
         if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Emiting failed signal!\n");
         if (dbg) fflush(stdout);
         emit connFailed(errno);
         if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Back from signal!\n");
         if (dbg) fflush(stdout);
      }
   }
   else
   {
      if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Accept succeded!\n");
      if (dbg) fflush(stdout);

      if (sna)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Disconnecting/deleting Accept Notifier!\n");
         if (dbg) fflush(stdout);
         QObject::disconnect(sna, SIGNAL(activated(int)), this, SLOT(readyAccept(int)));
         delete sna;
      }
      sna = NULL;

      n = fcntl(fd, F_GETFL, 0);
      n |= FNDELAY;
      fcntl(fd, F_SETFL, n); // SVR4 non-blocking incantation

      snr = new QSocketNotifier(fd, QSocketNotifier::Read, this);
      connect(snr, SIGNAL(activated(int)), this, SLOT(readyRead()));

      snw = new QSocketNotifier(fd, QSocketNotifier::Write, this);
      connect(snw, SIGNAL(activated(int)), this, SLOT(readyWrite()));
      snw->setEnabled( FALSE );
      connectFlag = TRUE;
      if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Emitting signal!\n");
      emit connAccepted(fd);
   }
   if (dbg) fprintf(stdout, "xSocketTCP::readyAccept():Exit\n");
   if (dbg) fflush(stdout);
}

struct sockaddr_in xSocketTCP::socketName()
{
   struct sockaddr_in addr;
   int x;

   addr.sin_family = AF_INET;
   addr.sin_port = 0;
   addr.sin_addr.s_addr = htonl(INADDR_ANY);

   if (fd >= 0)
   {
      x = sizeof(addr);
      getsockname(fd, (struct sockaddr *)&addr, &x);
   }
   return(addr);
}

bool xSocketTCP::accept(const char *pService)
{
   struct servent *se;
   int port, sock, rv = 0;
   char errBuf[80], *cp;

   if (dbg) fprintf(stdout, "xSocketTCP::accept():Enter\n");
   if (dbg) fflush(stdout);
   if (dbg) fprintf(stdout, "xSocketTCP::accept():getting Service\n");
   if (dbg) fflush(stdout);
   if (pService == NULL)
      pService = "0";
   if ((se = getservbyname(pService, "tcp")) != NULL)
      port = se->s_port;
   else
      port = htons((u_short)atoi(pService));
   if (dbg) fprintf(stdout, "xSocketTCP::accept():got port %d\n", port);
   if (dbg) fflush(stdout);

   acceptAddr.sin_family = AF_INET;
   acceptAddr.sin_port = port;
   acceptAddr.sin_addr.s_addr = htonl(INADDR_ANY);
   
   cp = NULL;
   if (dbg) fprintf(stdout, "xSocketTCP::accept():Calling socket()\n");
   if (dbg) fflush(stdout);
   if ((sock = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) >= 0)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::accept():Calling bind()\n");
      if (dbg) fflush(stdout);
      if (::bind(sock, (struct sockaddr *)&acceptAddr, 
                 sizeof(acceptAddr)) >= 0)
      {
         sna = new QSocketNotifier(sock, QSocketNotifier::Read, this);
         connect(sna, SIGNAL(activated(int)), this, SLOT(readyAccept(int)));

         if (dbg) fprintf(stdout, "xSocketTCP::accept():Calling listen()\n");
         if (dbg) fflush(stdout);
         ::listen(sock, 5);
         fd = sock;
         if (dbg) fprintf(stdout, "xSocketTCP::accept():fd = %d\n", fd);
         if (dbg) fflush(stdout);
      }
      else 
      {
         if (dbg) fprintf(stdout, "xSocketTCP::accept():Error binding name\n");
         if (dbg) fflush(stdout);
         cp = "Error Binding Socket";
      }
   }
   else
   {
      if (dbg) fprintf(stdout, "xSocketTCP::accept():Error opening socket\n");
      if (dbg) fflush(stdout);
      cp = "Error Opening Socket";
   }
   if (cp != NULL)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::accept():Error Found, starting message box!\n");
      if (dbg) fflush(stdout);
      sprintf(errBuf, "%s:%s", cp, strerror(errno));
      QMessageBox::message("Error", errBuf);
      rv = -1;
   }
   if (dbg) fprintf(stdout, "xSocketTCP::accept():Exit(%d)\n", rv);
   if (dbg) fflush(stdout);
   return(rv);
}

bool xSocketTCP::connectTo(const char *pHost, const char *pService)
{
   char errBuf[80], *cp;
   bool rv = TRUE;
   int x;

   if ((x = connectTCP(pHost, pService)) < 0)
   {
      switch (x)
      {
         case -1:
            cp = "Cannot find service entry";
            break;

         case -2:
            cp = "Cannot find host entry";
            break;

         case -3:
            cp = "Invalid protocol requested";
            break;

         case -4:
            cp = "Cannot create socket";
            break;

         case -5:
            cp = "Cannot connect to to host";
            break;

         default:
            cp = "Undertermined cause of error";
            break;
      }
      sprintf(errBuf, "%s:%s", cp, strerror(errno));
      QMessageBox::message("Error", errBuf);
      rv = FALSE;
   }
   if (dbg) fprintf(stdout, "xSocketTCP::connectTo():Exit\n");
   if (dbg) fflush(stdout);
   return(rv);
}

void xSocketTCP::sendToSocket(const char *pText)
{
   if (snw != NULL && snr != NULL)
   {
      if (connectFlag == TRUE && fd >= 0)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::sendText():Adding |%s| to writeBuffer\n", pText);
         if (dbg) fflush(stdout);
         writeBuffer += pText;
      }
      snw->setEnabled(!writeBuffer.isEmpty());
   }
}

static void timer(int sig)
{
   sig = 0;
   timedOut = TRUE;
   if (dbg) fprintf(stdout, "timer():Timer Expired\n");
   if (dbg) fflush(stdout);
   (void) signal(SIGALRM, timer);
}

void xSocketTCP::stopConnect()
{
   if (dbg) fprintf(stdout, "xSocketTCP::stopConnect():Enter\n");
   if (dbg) fflush(stdout);
   pTimer->stop();
   QObject::disconnect(pTimer, SIGNAL(timeout()), this, SLOT(retryConnect()));
   if (connectFlag == FALSE && fd >= 0)
      disconnectTCP();

   if (snc)
   {
      QObject::disconnect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
      delete snc;
      errno = ECONNABORTED;
      emit connFailed(errno);
   }
   snc = NULL;
   if (dbg) fprintf(stdout, "xSocketTCP::stopConnect():Exit\n");
   if (dbg) fflush(stdout);
}

void xSocketTCP::tryNextConnection()
{
   if (connectFlag == FALSE && fd >= 0)
   {
      retryFlag = TRUE;
      if (dbg) fprintf(stdout, "xSocketTCP::tryNextConnection():Enter\n");
      if (dbg) fflush(stdout);
      (void) alarm((unsigned)60); // 45 seconds to open connnection
      connIndex++;
      if (dbg) fprintf(stdout, "xSocketTCP::tryNextConnection():Trying Next Entry\n");
      if (dbg) fflush(stdout);
      close(fd);
      if ((fd = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) < 0)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::tryNextConnection():New Socket Failed!!!!\n");
         if (dbg) fflush(stdout);
         if (snc)
         {
            QObject::disconnect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
            delete snc;
         }
         snc = NULL;
         pTimer->stop();
         QObject::disconnect(pTimer, SIGNAL(timeout()), this, SLOT(retryConnect()));
         emit connFailed(errno);
         return;
      }
   }
   if (dbg) fprintf(stdout, "xSocketTCP::tryNextConnection():Exit!!!\n");
   if (dbg) fflush(stdout);
}

void xSocketTCP::retryConnect()
{
   retryConnect(fd);
}

void xSocketTCP::retryConnect(int x)
{
   int n;

   if (dbg > 2) fprintf(stdout, "xSocketTCP::retryConnect():Enter\n");
   if (dbg) fflush(stdout);
   x = x;
   if (timedOut)
   {
      if (connIndex >= 0)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Connection Timed Out!!!\n");
         if (dbg) fflush(stdout);
         if (snc)
         {
            QObject::disconnect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
            delete snc;
         }
         snc = NULL;
         close(fd);
         errno = ETIMEDOUT;
         pTimer->stop();
         QObject::disconnect(pTimer, SIGNAL(timeout()), this, SLOT(retryConnect()));
         emit connFailed(errno);
         (void) alarm((unsigned)0);
         return;
      }
      else
         connIndex++;
/*
      if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Time Out!! Trying Next Entry\n");
      close(fd);
      if ((sock = socket(AF_INET, SOCK_STREAM, PF_UNSPEC)) < 0)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Connection Timed Out!!!\n");
         if (snc)
         {
            QObject::disconnect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
            delete snc;
            snc = NULL;
         }
         pTimer->stop();
         emit connFailed(errno);
         return;
      }
      if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():New Socket Opened\n");
      fd = sock;
      (void) alarm((unsigned)60); // 45 seconds to open connnection
      connIndex++;
*/
   }

   if (phe != NULL && connIndex >= 0)
   {
      connAddr.sin_addr.s_addr = hostNames[connIndex];
      if (timedOut)
      {
         if (dbg > 2) fprintf(stdout, "xSocketTCP::retryConnect():Attempting Connection to %d: %s\n", connIndex,
                         inet_ntoa(connAddr.sin_addr));
         if (dbg) fflush(stdout);
         if (dbg > 2) fprintf(stdout, "xSocketTCP::retryConnect():Attempting Connection to %ld\n",
                         connAddr.sin_addr.s_addr);
         if (dbg) fflush(stdout);
      }
   }
   if ((phe == NULL && connIndex == 0) || connAddr.sin_addr.s_addr == 0)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Connection Timed Out!!!\n");
      if (dbg) fflush(stdout);
      if (snc)
      {
         QObject::disconnect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
         delete snc;
      }
      snc = NULL;
      errno = ETIMEDOUT;
      pTimer->stop();
      QObject::disconnect(pTimer, SIGNAL(timeout()), this, SLOT(retryConnect()));
      emit connFailed(errno);
      return;
   }
   if (timedOut || retryFlag)
   {
      if (retryFlag)
         retryFlag = FALSE;
      if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Emiting Connection Message w/ |%s|!!!\n",
                               inet_ntoa(connAddr.sin_addr));
      if (dbg) fflush(stdout);
      emit connecting(inet_ntoa(connAddr.sin_addr));
      (void) alarm((unsigned)60); // 45 seconds to open connnection
      if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Done Emiting Connection Message!!!\n");
      if (dbg) fflush(stdout);
   }
   timedOut = FALSE;

   if (pTimer->isActive() == FALSE)
   {
      pTimer->start(1000);
      QObject::connect(pTimer, SIGNAL(timeout()), this, SLOT(retryConnect()));
   }

   if ((n = fcntl(fd, F_GETFL, 0)) < 0)
      return;
   n |= FNDELAY;

   fcntl(fd, F_SETFL, n); // SVR4 non-blocking incantation

   if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Calling ::connect!!!\n");
   if (dbg) fflush(stdout);
   if (::connect(fd, (struct sockaddr *)&connAddr, sizeof(connAddr)) >= 0)
   {
      pTimer->stop();
      QObject::disconnect(pTimer, SIGNAL(timeout()), this, SLOT(retryConnect()));
      if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Connect complete!!!\n");
      if (dbg) fflush(stdout);
      (void) alarm((unsigned)0);
      unfinished = FALSE;
   
      if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():Connection Suceeded!!!\n");
      if (dbg) fflush(stdout);
      if (snc)
      {
         QObject::disconnect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
         delete snc;
      }
      snc = NULL;

      snr = new QSocketNotifier(fd, QSocketNotifier::Read, this);
      connect(snr, SIGNAL(activated(int)), this, SLOT(readyRead()));

      snw = new QSocketNotifier(fd, QSocketNotifier::Write, this);
      connect(snw, SIGNAL(activated(int)), this, SLOT(readyWrite()));
      snw->setEnabled( FALSE );

      connectFlag = TRUE;

      if (dbg) fprintf(stdout, "xSocketTCP::retryConnect():connFlag = %d, fd = %d!!!\n", connectFlag, fd);
      if (dbg) fflush(stdout);
      emit connAccepted(fd);
   }
   else
   {
      if (errno != EWOULDBLOCK &&
          errno != EINPROGRESS &&
          errno != EALREADY &&
          errno != EAGAIN)
      {
         close(fd);
         perror("xSocketTCP::retryConnect():Connection Failed!!!\n");
         if (snc)
         {
            QObject::disconnect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
            delete snc;
         }
         snc = NULL;
         pTimer->stop();
         QObject::disconnect(pTimer, SIGNAL(timeout()), this, SLOT(retryConnect()));
         emit connFailed(errno);
      }
   }
   if (dbg > 2) fprintf(stdout, "xSocketTCP::retryConnect():Exit\n");
   if (dbg) fflush(stdout);
}

int xSocketTCP::connectTCP(const char *pHost, const char *pService)
{
   struct servent *se = NULL;
   int port, *pAddr;
   int x, sock;

// first find some elementary numbers
   if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():getting Service Entry\n");
   if (dbg) fflush(stdout);

   memset((void *)&connAddr, '\0', sizeof(connAddr));
   if ((se = getservbyname(pService, "tcp")) != NULL)
      port = se->s_port;
   else if ((port = htons((u_short)atoi(pService))) == 0)
      return(-1);

   phe = NULL;
   if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():getting Host Entry\n");
   if (dbg) fflush(stdout);
   if ((phe = gethostbyname(pHost)) == NULL)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():Maybe by Dotted Quad\n");
      if (dbg) fflush(stdout);
      if ((connAddr.sin_addr.s_addr = inet_addr(pHost)) == INADDR_NONE)
      {
         if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():Must be INET address number\n");
         if (dbg) fflush(stdout);
         if ((connAddr.sin_addr.s_addr = atoi(pHost)) == 0)
            return(-2);
      }
      if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():Found Lone address: %ld\n",
                               connAddr.sin_addr.s_addr);
      if (dbg) fflush(stdout);
      connIndex = -2;
   }
   else
   {
      if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():Was a name after all!!!\n");
      if (dbg) fflush(stdout);
      for (x = 0, pAddr = (int*)phe->h_addr_list[0];
           x < (int)((sizeof(hostNames) / sizeof(hostNames[0])) - 1) && 
                  *pAddr != 0; x++, pAddr++)
      {
         hostNames[x] = *pAddr;
         connAddr.sin_addr.s_addr = hostNames[x];
         if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():Found address: %ld\n",
                                   connAddr.sin_addr.s_addr);
         if (dbg) fflush(stdout);
      }
      hostNames[x] = 0;
      connIndex = -1;
   }

   sock = -1;
   connAddr.sin_family = AF_INET;
   connAddr.sin_port = port;
   sock = socket(AF_INET, SOCK_STREAM, PF_UNSPEC);
   if (sock < 0)
      return(-5);
   fd = sock;

   if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():Connecting socket\n");
   if (dbg) fflush(stdout);
   snc = new QSocketNotifier(sock, QSocketNotifier::Write, this);
   connect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
   (void) signal(SIGALRM, timer);
   timedOut = TRUE;
   retryConnect(sock);
   if (dbg) fprintf(stdout, "xSocketTCP::connectTCP():Exit!!!\n");
   if (dbg) fflush(stdout);
   return(0);
}

int xSocketTCP::disconnectTCP()
{
   int rv = 0;

   if (dbg) fprintf(stdout, "xSocketTCP::disconnectTCP():Enter\n");
   if (dbg) fflush(stdout);
   if (snw != NULL)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::disconnectTCP():Disconnecting snw\n");
      if (dbg) fflush(stdout);
      QObject::disconnect(snr, SIGNAL(activated(int)), this, SLOT(readyRead()));
      delete snw;
   }
   snw = NULL;
   if (snr != NULL)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::disconnectTCP():Disconnecting snr\n");
      if (dbg) fflush(stdout);
      QObject::disconnect(snr, SIGNAL(activated(int)), this, SLOT(readyWrite()));
      delete snr;
   }
   snr = NULL;
   if (sna != NULL)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::disconnectTCP():Disconnecting sna\n");
      if (dbg) fflush(stdout);
      QObject::disconnect(sna, SIGNAL(activated(int)), this, SLOT(readyAccept(int)));
      delete sna;
   }
   sna = NULL;
   if (snc != NULL)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::disconnectTCP():Disconnecting snc\n");
      if (dbg) fflush(stdout);
      QObject::disconnect(snc, SIGNAL(activated(int)), this, SLOT(retryConnect(int)));
      delete snc;
   }
   snc = NULL;
   if (fd >= 0)
   {
      if (dbg) fprintf(stdout, "xSocketTCP::disconnectTCP():Shutting down Socket!\n");
      if (dbg) fflush(stdout);
      rv = shutdown(fd, 2);
   }
   if (dbg) fprintf(stdout, "xSocketTCP::disconnectTCP():Calling readyRead()\n");
   if (dbg) fflush(stdout);
   connectFlag = FALSE;
   fd = -1;
   if (dbg) fprintf(stdout, "xSocketTCP::disconnectTCP():Exit\n");
   if (dbg) fflush(stdout);
   return(rv);
}

#include "xSocket.moc"

