/***************************************************************************
**    xSocket.h  $Revision: 1.5 $ - $Name:  $ 
**    Class to manipulate sockets
**
**    Copyright (C) 1996 Joseph Croft <jcroft@unicomp.net>
**
**    This library is free software; you can redistribute it and/or
**    modify it under the terms of the GNU Library General Public
**    License as published by the Free Software Foundation; either
**    version 2 of the License, or (at your option) any later version.
**
**    This library is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**    Library General Public License for more details.
**
**    You should have received a copy of the GNU Library General Public
**    License along with this library; if not, write to the Free
**    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#ifndef _xSOCKET_H
#define _xSOCKET_H

#include <qobject.h>
#include <qstring.h>
#include <qsocknot.h>
#include <qtimer.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>

class xSocketTCP : public QObject
{
   Q_OBJECT
   
public:
   xSocketTCP(QObject *pParent = NULL, char *pName = NULL);
   ~xSocketTCP();
   
   virtual bool   accept(const char *pService = NULL);
   virtual bool   connectTo(const char *pHost, const char *pService);
   virtual int    disconnectTCP();
   bool           isConnected() { return(connectFlag); };
   struct sockaddr_in socketName();
   
signals:
   void socketError(int err);
   void readFromSocket(char *buf);
   void connecting(const char *inet_addr);
   void socketClosed();
   void connAccepted(int sock);
   void connFailed(int sock);

public slots:
   void sendToSocket(const char *pText);
   void tryNextConnection();
   void stopConnect();
   
private slots:
   void readyRead();
   void readyWrite();
   void readyAccept(int sock);
   void retryConnect(int sock);
   void retryConnect();

protected:
   FILE     *dbf;
   
private:
   QTimer            *pTimer;
   QString           writeBuffer;
   long              hostNames[50];
   int               counter;
   int               fd;
   int               connIndex;
   FILE              *pFdIn;
   char              *pHost;
   char              *pService;
   bool              connectFlag;
   QSocketNotifier   *snr;
   QSocketNotifier   *snw;
   QSocketNotifier   *sna;
   QSocketNotifier   *snc;
   struct hostent    *phe;
   struct sockaddr_in acceptAddr;
   struct sockaddr_in connAddr;
   int connectTCP(const char *pHost, const char *pService);
};

#define ESOCK_BadWrite        1
#define ESOCK_BadRead         2
#endif
