/***************************************************************************
**    qtIrc.cpp  $Revision: 1.14 $ - $Name:  $ 
**    IRC Client Main Line code
**
** Copyright (C) 1995, 1996  Joseph Croft <jcroft@unicomp.net>  
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
 ***************************************************************************/
#include <qapp.h>

#include <qpainter.h>
#include <qwindefs.h>
#include <qfontinf.h>
#include <qpixmap.h>
#include <qmsgbox.h>
#include <qbuffer.h>
#include <qimage.h>
#include <qstring.h>
#include <qobject.h>
#include <qevent.h>
#include <qtimer.h>
#include <qkeycode.h>

#include <qlined.h>
#include <qlabel.h>
#include <qdialog.h>

#include <stdlib.h>
#include <stdio.h>

#include "xIrcServerQuery.h"
#include "xIrcChannelQuery.h"
#include "xIrcNickQuery.h"
#include "xLineEditQuery.h"
#include "xIrcConnect.h"
#include "xDefaults.h"
#include "xIrcNickCombo.h"

static int dbg = 0;

xChannelQuery *ChanQuery;
xIrcNickQuery *NickQuery;
xLineEditQuery *KickQuery;
xLineEditQuery *QuitQuery;
xServerQuery *ServQuery;

xDefaults Defaults;
QApplication *pApp = NULL;
QEvent qEvt(-1);
static char defEscapes[][2] =
{
   { 'b', '\x02' },
   { 'u', '\x1f' }
};

char *version = "$Name:  $";

class MyApp : public QApplication
{
public:
   MyApp(int argc, char **argv);
   bool notify(QObject *pRcvr, QEvent *pEvt);
};

MyApp::MyApp(int argc, char **argv) :
         QApplication(argc, argv)
{
   if (dbg) fprintf(stdout, "MyApp::MyApp()!!!\n");
   if (dbg) fflush(stdout);
}

bool MyApp::notify(QObject *pRcvr, QEvent *pEvt)
{
   return(QApplication::notify(pRcvr, pEvt));
}

int main(int argc, char **argv)
{
   QColor fg, bg;
   FILE *pfd;
   char *cp, *cp1;
   unsigned long rgbColor;

   MyApp a( argc, argv );
   pApp = &a;

   if (dbg) fprintf(stdout, "main():Opening defaults\n");   
   if (dbg) fflush(stdout);
   pfd = fopen("xIrc.defaults", "r");
   Defaults.setEscapes(defEscapes);
   Defaults.load(pfd, NULL);
   if (dbg) Defaults.show();
   
   if (dbg) fprintf(stdout, "main():Setting Colors and Pallete\n");   
   if (dbg) fflush(stdout);
   if ((cp = (char *)Defaults.get("FRAME_BG_COLOR_RGB")) == NULL)
   {
      if ((cp = (char *)Defaults.get("FRAME_BG_COLOR_NAME")) == NULL)
         bg.setRgb(0xc3c3c3);	// Motif gray
      else
         bg.setNamedColor(cp);
   }
   else
   {
      rgbColor = strtol(cp, &cp1, 0);
      if (cp != cp1)
         bg.setRgb(rgbColor);
      else
         bg.setRgb(0xc3c3c3);	// Motif gray 
   }

   if ((cp = (char *)Defaults.get("FRAME_FG_COLOR_RGB")) == NULL)
   {
      if ((cp = (char *)Defaults.get("FRAME_FG_COLOR")) == NULL)
         cp = "black";
      fg.setNamedColor(cp);
   }
   else
   {
      rgbColor = strtol(cp, &cp1, 0);
      if (cp != cp1)
         fg.setRgb(rgbColor);
      else
         fg.setNamedColor("black");	// kind of blue
   }

   QColorGroup g( fg, bg, bg.light(), bg.dark(), bg.dark(120),	fg, white );
   QPalette p( g, g, g );

   a.setPalette( p );

   if (dbg) fprintf(stdout, "main():Opening channel Query\n");   
   if (dbg) fflush(stdout);
   ChanQuery = new xChannelQuery(NULL, "Channels & People");
   if (dbg) fprintf(stdout, "main():NickQuery\n");   
   if (dbg) fflush(stdout);
   NickQuery = new xIrcNickQuery(NULL, "Nickname");
   if (dbg) fprintf(stdout, "main():KickQuery\n");   
   if (dbg) fflush(stdout);
   KickQuery = new xLineEditQuery("Enter Kick Message", "", NULL, "Kick Message",
                                   "KICKMESSAGE", &Defaults);
   if (dbg) fprintf(stdout, "main():QuitQuery\n");   
   if (dbg) fflush(stdout);
   QuitQuery = new xLineEditQuery("Enter Quit Message", "", NULL, "Quit Message",
                                   "QUITMESSAGE", &Defaults);
   if (dbg) fprintf(stdout, "main():ServQuery\n");   
   if (dbg) fflush(stdout);
   ServQuery = new xServerQuery(NULL, "Server Selection");

   if (dbg) fprintf(stdout, "main():xIrcConnect\n");   
   if (dbg) fflush(stdout);
   xIrcConnect *pTWindow = new xIrcConnect;
   
   /*
   QMenuBar *pMenu = new QMenuBar(pTWindow);
   pMenu->insertItem("Server", pTWindow, SLOT(connectSocket()), CTRL+ALT+Key_C);
   pMenu->insertItem("Channel", 1, 1);
   pMenu->insertItem("Menu3", 2, 2);
   
   pTWindow->attachMenu(pMenu);
   pMenu->show();
   
   while (ServQuery->exec() == QDialog::Rejected);
   fprintf(stderr, "Selected Server = |%s|\n\r", ServQuery->server());
   if (dbg) fflush(stdout);
   fprintf(stderr, "Selected Port = |%s|\n\r", ServQuery->port());
   if (dbg) fflush(stdout);
   
   while (NickQuery->exec() == QDialog::Rejected);
   fprintf(stderr, "Selected Nick = |%s|\n\r", NickQuery->editText());
   if (dbg) fflush(stdout);
   */   
   
   if (dbg) fprintf(stdout, "main():Showing/Setting/executing app\n");
   if (dbg) fflush(stdout);
   pTWindow->show();
   a.setMainWidget( pTWindow );
   for (;;)
   {
      if (NickQuery->exec() == QDialog::Rejected)
         QMessageBox::message("Error", "You must choose a NickName");
      else
         break;
   }
   pTWindow->newServer();
   a.exec();
   if (pTWindow)
   {
      if (dbg) fprintf(stdout, "main():Deleting pTWindow\n");
      if (dbg) fflush(stdout);
      delete pTWindow;
   }
   if (ChanQuery)
   {
      if (dbg) fprintf(stdout, "main():Deleting ChanQuery\n");
      if (dbg) fflush(stdout);
      delete ChanQuery;
   }
   if (NickQuery)
   {
      if (dbg) fprintf(stdout, "main():Deleting NickQuery\n");
      if (dbg) fflush(stdout);
      delete NickQuery;
   }
   if (KickQuery)
   {
      if (dbg) fprintf(stdout, "main():Deleting KickQuery\n");
      if (dbg) fflush(stdout);
      delete KickQuery;
   }
   if (QuitQuery)
   {
      if (dbg) fprintf(stdout, "main():Deleting QuitQuery\n");
      if (dbg) fflush(stdout);
      delete QuitQuery;
   }
   if (ServQuery)
   {
      if (dbg) fprintf(stdout, "main():Deleting ServQuery\n");
      if (dbg) fflush(stdout);
      delete ServQuery;
   }
   if (dbg) fprintf(stdout, "main():Exiting!\n");
   if (dbg) fflush(stdout);
}
